package ru.dbotthepony.mc.otm.capability.matter

import net.minecraft.util.Mth
import net.minecraftforge.common.capabilities.ICapabilityProvider
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.core.ImpreciseFraction
import ru.dbotthepony.mc.otm.core.RGBAColor
import ru.dbotthepony.mc.otm.core.orNull
import kotlin.math.roundToInt

interface IMatterHandler {
	val storedMatter: ImpreciseFraction
	val maxStoredMatter: ImpreciseFraction

	fun receiveMatterOuter(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction
	fun receiveMatterInner(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction
	fun extractMatterOuter(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction
	fun extractMatterInner(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction

	val direction: MatterDirection
	val missingMatter: ImpreciseFraction
		get() = maxStoredMatter.minus(storedMatter).moreThanZero()

	val allowsExtract: Boolean
		get() = direction != MatterDirection.RECEIVE

	val allowsReceive: Boolean
		get() = direction != MatterDirection.EXTRACT
}

fun IMatterHandler.getBarWidth(): Int {
	return ((storedMatter / maxStoredMatter).toFloat().coerceAtLeast(0f).coerceAtMost(1f) * 13f).roundToInt()
}

fun IMatterHandler.getBarColor(): Int {
	return RGBAColor.LOW_MATTER.linearInterpolation((storedMatter / maxStoredMatter).toFloat(), RGBAColor.FULL_MATTER).toInt()
}

val ICapabilityProvider.matter: IMatterHandler? get() = getCapability(MatteryCapability.MATTER).orNull()
