package ru.dbotthepony.mc.otm.capability.matter

import net.minecraft.nbt.CompoundTag
import net.minecraftforge.common.util.INBTSerializable
import net.minecraftforge.common.util.LazyOptional
import ru.dbotthepony.mc.otm.ConciseBalanceValues
import ru.dbotthepony.mc.otm.VerboseBalanceValues
import ru.dbotthepony.mc.otm.core.ImpreciseFraction
import ru.dbotthepony.mc.otm.container.set
import ru.dbotthepony.mc.otm.core.set

open class MatterHandlerImpl @JvmOverloads constructor(
	protected val listener: Runnable?,
	override val direction: MatterDirection,
	protected val maxStoredMatterSupplier: () -> ImpreciseFraction,
	protected val maxReceiveSupplier: () -> ImpreciseFraction? = { null },
	protected val maxExtractSupplier: () -> ImpreciseFraction? = maxReceiveSupplier
) : IMatterHandler, INBTSerializable<CompoundTag> {
	constructor(
		listener: Runnable?,
		direction: MatterDirection,
		maxStoredMatter: ImpreciseFraction,
		maxReceive: ImpreciseFraction? = null,
		maxExtract: ImpreciseFraction? = null,
	) : this(listener, direction, { maxStoredMatter }, { maxReceive }, { maxExtract })

	constructor(
		listener: Runnable?,
		direction: MatterDirection,
		values: ConciseBalanceValues
	) : this(listener, direction, values::capacity, values::throughput, values::throughput)

	constructor(
		listener: Runnable?,
		direction: MatterDirection,
		values: VerboseBalanceValues
	) : this(listener, direction, values::capacity, values::receive, values::extract)

	val maxReceive get() = maxReceiveSupplier.invoke()
	val maxExtract get() = maxExtractSupplier.invoke()

	override val maxStoredMatter: ImpreciseFraction
		get() = maxStoredMatterSupplier.invoke()

	override var storedMatter = ImpreciseFraction.ZERO
		protected set

	private var handler = LazyOptional.of<IMatterHandler> { this }

	fun invalidate() {
		handler.invalidate()
	}

	fun revive() {
		handler = LazyOptional.of { this }
	}

	fun get(): LazyOptional<IMatterHandler> {
		return handler
	}

	open fun canReceiveAll(value: ImpreciseFraction): Boolean {
		return maxStoredMatter >= value && storedMatter + value <= maxStoredMatter
	}

	override fun receiveMatterOuter(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
		if (direction === MatterDirection.EXTRACT)
			return ImpreciseFraction.ZERO

		return receiveMatterInner(howMuch, simulate)
	}

	override fun receiveMatterInner(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
		val new: ImpreciseFraction

		if (maxReceive == null) {
			new = (storedMatter + howMuch).coerceAtMost(maxStoredMatter)
		} else {
			new = (storedMatter + howMuch.coerceAtMost(maxReceive!!)).coerceAtMost(maxStoredMatter)
		}

		val diff = new - storedMatter

		if (!simulate && new != storedMatter) {
			storedMatter = new
			listener?.run()
		}

		return diff
	}

	override fun extractMatterOuter(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
		if (direction === MatterDirection.RECEIVE)
			return ImpreciseFraction.ZERO

		return extractMatterInner(howMuch, simulate)
	}

	override fun extractMatterInner(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
		val new: ImpreciseFraction

		if (maxExtract == null) {
			new = (storedMatter - howMuch).moreThanZero()
		} else {
			new = (storedMatter - howMuch.coerceAtMost(maxExtract!!)).moreThanZero()
		}

		val diff = storedMatter - new

		if (!simulate && new != storedMatter) {
			storedMatter = new
			listener?.run()
		}

		return diff
	}

	override fun serializeNBT(): CompoundTag {
		return CompoundTag().also {
			it["stored"] = storedMatter.serializeNBT()
			//it["max_storage"] = maxStoredMatter.serializeNBT()
			//if (maxReceive != null) it["max_receive"] = maxReceive.serializeNBT()
			//if (maxExtract != null) it["max_extract"] = maxExtract.serializeNBT()
		}
	}

	override fun deserializeNBT(tag: CompoundTag) {
		storedMatter = ImpreciseFraction.deserializeNBT(tag["stored"])
		//maxStoredMatter = ImpreciseFraction.deserializeNBT(tag["max_storage"])
		//maxReceive = if (tag.contains("max_receive")) ImpreciseFraction.deserializeNBT(tag["max_receive"]) else null
		//maxExtract = if (tag.contains("max_extract")) ImpreciseFraction.deserializeNBT(tag["max_extract"]) else null
	}
}