package ru.dbotthepony.mc.otm.client.screen.panels.buttons

import com.mojang.blaze3d.vertex.PoseStack
import net.minecraft.client.gui.screens.Screen
import ru.dbotthepony.mc.otm.client.playGuiClickSound
import ru.dbotthepony.mc.otm.client.render.AbstractSkinElement
import ru.dbotthepony.mc.otm.client.screen.panels.EditablePanel

@Suppress("PropertyName")
abstract class RectangleButtonPanel<out S : Screen>(
	screen: S,
	parent: EditablePanel<*>?,
	x: Float = 0f,
	y: Float = 0f,
	width: Float,
	height: Float,
	val onPress: ((clickButton: Int) -> Unit)? = null,
) : EditablePanel<S>(screen, parent, x, y, width, height) {
	protected var pressed = false

	protected open fun onClick(clickButton: Int) {
		onPress?.invoke(clickButton)
	}

	abstract val PRESSED: AbstractSkinElement
	abstract val HOVERED: AbstractSkinElement
	abstract val IDLE: AbstractSkinElement
	abstract val DISABLED: AbstractSkinElement

	var isDisabled = false
		set(value) {
			if (field != value) {
				if (!value) {
					pressed = false
					grabMouseInput = false
				}

				field = value
			}
		}

	override fun innerRender(stack: PoseStack, mouseX: Float, mouseY: Float, partialTick: Float) {
		if (isDisabled) {
			DISABLED.render(stack, 0f, 0f, width, height)
		} else if (pressed) {
			PRESSED.render(stack, 0f, 0f, width, height)
		} else if (isHovered) {
			HOVERED.render(stack, 0f, 0f, width, height)
		} else {
			IDLE.render(stack, 0f, 0f, width, height)
		}
	}

	override fun mouseClickedInner(x: Double, y: Double, button: Int): Boolean {
		if (!isDisabled) {
			if (!pressed) {
				playGuiClickSound()
			}

			if (tryToGrabMouseInput()) {
				pressed = true
			}
		}

		return true
	}

	override fun mouseReleasedInner(x: Double, y: Double, button: Int): Boolean {
		if (!isDisabled && pressed) {
			pressed = false

			if (isHovered) {
				onClick(button)
			}
		}

		grabMouseInput = false

		return true
	}
}
