package ru.dbotthepony.mc.otm.core

import io.netty.handler.codec.EncoderException
import net.minecraft.nbt.CompoundTag
import net.minecraft.nbt.NbtAccounter
import net.minecraft.nbt.NbtIo
import net.minecraft.world.item.Item
import net.minecraft.world.item.ItemStack
import net.minecraftforge.registries.ForgeRegistries
import net.minecraftforge.registries.ForgeRegistry
import java.io.*
import java.math.BigDecimal
import java.math.BigInteger

// But seriously, Mojang, why would you need to derive from ByteBuf directly, when you can implement
// your own InputStream and OutputStream, since ByteBuf is meant to be operated on most time like a stream anyway?

// netty ByteBuf -> netty ByteBufInputStream -> Minecraft FriendlyInputStream

fun OutputStream.writeNbt(value: CompoundTag) {
	try {
		NbtIo.write(value, if (this is DataOutputStream) this else DataOutputStream(this))
	} catch (ioexception: IOException) {
		throw EncoderException(ioexception)
	}
}

fun InputStream.readNbt(accounter: NbtAccounter = NbtAccounter.UNLIMITED): CompoundTag {
	return try {
		NbtIo.read(if (this is DataInputStream) this else DataInputStream(this), accounter)
	} catch (ioexception: IOException) {
		throw EncoderException(ioexception)
	}
}

fun OutputStream.writeItem(itemStack: ItemStack, limitedTag: Boolean = true) {
	if (itemStack.isEmpty) {
		write(0)
	} else {
		write(1)
		val id = (ForgeRegistries.ITEMS as ForgeRegistry<Item>).getID(itemStack.item)

		writeInt(id)
		writeInt(itemStack.count)

		var compoundtag: CompoundTag? = null

		if (itemStack.item.isDamageable(itemStack) || itemStack.item.shouldOverrideMultiplayerNbt()) {
			compoundtag = if (limitedTag) itemStack.shareTag else itemStack.tag
		}

		write(if (compoundtag != null) 1 else 0)

		if (compoundtag != null) {
			writeNbt(compoundtag)
		}
	}
}

fun InputStream.readItem(): ItemStack {
	if (read() == 0) {
		return ItemStack.EMPTY
	}


	val item = (ForgeRegistries.ITEMS as ForgeRegistry<Item>).getValue(readInt())
	val itemStack = ItemStack(item, readInt())

	if (read() != 0) {
		itemStack.readShareTag(readNbt())
	}

	return itemStack
}

fun OutputStream.writeBigDecimal(value: BigDecimal) {
	writeInt(value.scale())
	val bytes = value.unscaledValue().toByteArray()
	writeVarIntLE(bytes.size)
	write(bytes)
}

fun InputStream.readBigDecimal(): BigDecimal {
	val scale = readInt()
	val size = readVarIntLE()
	val bytes = ByteArray(size)
	read(bytes)
	return BigDecimal(BigInteger(bytes), scale)
}

